use crate::dynamic_data::{
    BattleStatCalculator, DamageLibrary, DynamicLibrary, DynamicLibraryImpl, MiscLibrary, ScriptResolver,
};
use crate::ffi::{IdentifiablePointer, OwnedPtr};
use crate::static_data::StaticData;
use std::sync::Arc;

/// Instantiates a new DynamicLibrary with given parameters.
#[no_mangle]
extern "C" fn dynamic_library_new(
    static_data: OwnedPtr<Box<dyn StaticData>>,
    stat_calculator: OwnedPtr<Box<dyn BattleStatCalculator>>,
    damage_library: OwnedPtr<Box<dyn DamageLibrary>>,
    misc_library: OwnedPtr<Box<dyn MiscLibrary>>,
    script_resolver: OwnedPtr<Box<dyn ScriptResolver>>,
) -> IdentifiablePointer<Arc<dyn DynamicLibrary>> {
    unsafe {
        let a: Arc<dyn DynamicLibrary> = Arc::new(DynamicLibraryImpl::new(
            *Box::from_raw(static_data),
            *Box::from_raw(stat_calculator),
            *Box::from_raw(damage_library),
            *Box::from_raw(misc_library),
            *Box::from_raw(script_resolver),
        ));
        a.into()
    }
}
